import QUB
from Buttons import *
from qubtree import Node
import win32com.client
from pythoncom import CoInitialize, CoUninitialize

### -------------------------------------------------------------------------
### Find pulses as transition below PulseLevel followed by transition above pulse level
### then add a list segment for each pulse 
def Jeff(props):

    nLevel = props['PulseLevel']
    iPulseChannel = props['PulseChannel']
    iDataChannel = props['DataChannel']
    iLeadSkip = props['LeadingSkip']
    iTrailSkip = props['TrailingSkip']
    iBaseline = props['BaselinePts']
    iExcelMarginR = props['ExcelMarginR']
    iExcelMarginL = props['ExcelMarginL']
    iPulseCount = props['PulseCount']
    if props['ExportDetails'] == 'yes':
        lDetail=True
    else:
        lDetail=False
    
    QUB.Report( 'Pulse averages' )

    ##-- Prepare excel 
    excel = win32com.client.Dispatch("Excel.Application")
    excel.Visible = 1
    book = excel.Workbooks.Add(-4167) # XLWBatWorksheet
    sheet = book.Worksheets.Item(1)
    cells = sheet.Cells
    
    ##-- Get The Data - dataStim and dataResp
    alldata = QUB.DataView.currData
    iSaveChannel=alldata.activeChannel
    alldata.activeChannel=iPulseChannel
    dataStim=QUB.getChosenData()
    alldata.activeChannel=iDataChannel
    dataResp=QUB.getChosenData()
    alldata.activeChannel=iSaveChannel

    ##-- Count the Segments
    iDetailCol=1
    if lDetail : 
        segStim = dataStim.find('DataSet').find('Segment')
        segResp = dataResp.find('DataSet').find('Segment')
        while (not segStim.isNull) and (not segResp.isNull) :
            ##-- Next Segment 
            segStim = segStim.next('Segment')
            segResp = segResp.next('Segment')
            iDetailCol=iDetailCol+1
            iDetailRow=1

    ##-- scan the data for pulses, add to list 
    segStim = dataStim.find('DataSet').find('Segment')
    segResp = dataResp.find('DataSet').find('Segment')
    iSeg=0
    iMaxPts=0
    while (not segStim.isNull) and (not segResp.isNull) :
        aCount=[]
        aSum=[]
        while len(aSum) < iExcelMarginR :
            aSum.append(0)
            aCount.append(0)
        iPulses=0
    
        ##-- Load data segment 
        QUB.fillData(segStim)
        chanStim = segStim.find('Channel')
        iPtsStim = chanStim.data.count
        
        QUB.fillData(segResp)
        chanResp = segResp.find('Channel')
        iPtsResp = chanResp.data.count
        
        if iPtsStim<iPtsResp : 
            iPtsStim=iPtsResp        
            
        ##-- calc total
        nSum = 0
        nBase = 0 
        iPts = 0
        iPulse = 0
        iStart = -1 * iTrailSkip
        for iPoint in range(iPtsStim):
            ## start a new pulse
            if (iPulse==0) and (chanStim.data[iPoint]>nLevel) :
                iPulses=iPulses+1
                if (iPulses<=iPulseCount) and lDetail :
                    iDetailCol=iDetailCol+1
                    iDetailRow=1
                    cells(1,iDetailCol).Value = 'S '+str(iSeg)+ ' P '+str(iPulses)

                iPulse=1
                iStart=iPoint
                nBase=0
                iBasePts=0
                for iBP in range( max(0,iPoint-iBaseline), iPoint ) :
                    nBase = nBase + chanResp.data[iBP]
                    iBasePts = iBasePts + 1
                if iBasePts > 0 :
                    nBase = nBase / iBasePts
                for iPT in range( iExcelMarginL ) :
                    if iPoint-iExcelMarginL+iPT >= 0 :
                        aSum[iPT]=aSum[iPT]+chanResp.data[iPoint-iExcelMarginL+iPT]-nBase
                        aCount[iPT]=aCount[iPT]+1
            ## end a pulse
            if (iPulse==1) and (chanStim.data[iPoint]<=nLevel) : 
                iPulse=0
                iStart=iPoint
            ## good data within pulse
            if iPulses>0 and iPulses<=iPulseCount :
                d = chanResp.data[iPoint]-nBase
                if ((iPulse==1) and (iPoint>=iStart+iLeadSkip)) or ((iPulse==0) and (iPoint<iStart+iTrailSkip)) : 
                    nSum = nSum + d
                    iPts = iPts+1
                if (iPulse==1) or (iPoint<iStart+iExcelMarginR) :
                    if iPulse==1 :
                        iXLPt=iExcelMarginL+iPoint-iStart
                    else :
                        iXLPt=iXLPt + 1

                    if lDetail :
                        cells( iXLPt+2, iDetailCol).Value = d

                    while iXLPt >= len(aSum) :
                        aSum.append(0)
                        aCount.append(0)
                    aSum[iXLPt]=aSum[iXLPt] + d
                    aCount[iXLPt]=aCount[iXLPt]+1

                
        ##-- output results 
        if iPts==0 : 
            QUB.Report( '000' )
        else : 
            QUB.Report( str(nSum/iPts) )

        ##-- Output to excel 
        cells( 1, iSeg+1 ).Value = 'Seg '+str(iSeg)
        if len(aSum)>0 :
            iMaxPts=max(iMaxPts,len(aSum))
            for iPt in range( len(aSum) ) :
                if aCount[iPt]>0 :
                    cells( iPt+2, iSeg+1).Value = aSum[iPt]/aCount[iPt]

        ##-- Next Segment 
        segStim = segStim.next('Segment')
        segResp = segResp.next('Segment')
        iSeg=iSeg+1
        
    sheet.ChartObjects().Add(25,25,500,300)
    chart = sheet.ChartObjects(1).Chart
    chart.ChartType = 65 # xlLineMarkers  (xlLine=4) (xlLineStacked=63)
    rngChart = sheet.Range( sheet.Cells(1,1), sheet.Cells(1+iMaxPts,iSeg-1) )
    chart.SeriesCollection().Add( rngChart )
    ##chart.SeriesCollection(1).Border.Color = 0x999999
    ##chart.SeriesCollection(1).Border.Weight = 0x2 # xlThin
    excel.Visible = 1

AddTaskButton( 'Jeff', # button text
               Jeff,   # function
               'Jeff',       # short text for small buttons
               0,            # show settings dialog on left-click?
               1,            # show settings dialog on right-click?
               "Pulse averages",   # tool-tip
               "Pulse averages",   # description in "Edit Buttons"
               "Pulse averages:",  # caption for the settings dialog
               [TextField("PulseLevel", "Pulse when > :", "-25.0", acceptFloat ),
                TextField("PulseChannel", "Pulse Channel:", "1", acceptIntGreaterThan(-1)),
                TextField("PulseCount", "Pulse Count:", "5", acceptIntGreaterThan(0)),
                TextField("DataChannel", "Data Channel:", "0", acceptIntGreaterThan(-1)),
                TextField("LeadingSkip", "Leading Points to skip :", "0", acceptIntGreaterThan(-1)),
                TextField("TrailingSkip", "Trailing Points to include :", "0", acceptIntGreaterThan(-1)),
                TextField("BaselinePts", "Pts preceding pulse to calc baseline :", "20", acceptIntGreaterThan(0)),
                TextField("ExcelMarginL", "Pts before pulse to export :", "100", acceptIntGreaterThan(-1)),
                TextField("ExcelMarginR", "Pts after pulse to export :", "100", acceptIntGreaterThan(-1)),
                DialogRadioGroup("ExportDetails", "Export Details:", ['yes', 'no'], 'yes'),
                ],
               'Lists' )


